                   (*+------------------------------------+*)
                   (*|  TP sur les expressions formelles  |*)
                   (*+------------------------------------+*)

#directory "/home/quercia/lib";; (*  adapter en fonction des circonstances *)
#open "arbres";;
load_object "arbres";;

type formule =
| Const of float                    (* constantes *)
| Var of string                     (* variables  *)
| Fct of string * formule           (* (fonction, argument) *)
| Bin of string * formule * formule (* (oprateur, op-gauche, op-droit) *)
;;

(* impression *)
let rec g_arbre = function
| Const(c)    -> G_noeud(string_of_float c, [])
| Var(x)      -> G_noeud(x, [])
| Fct(f,u)    -> G_noeud(f, [g_arbre u])
| Bin(op,u,v) -> G_noeud(op,[g_arbre u; g_arbre v])
;;
let print_formule(f) = print_string_g_arbre(g_arbre f);;
install_printer "print_formule";;

(* analyseur d'entre *)
#open "genlex";;
let fonctions = ["sin"; "cos"; "exp"; "ln"; "tan"; "opp"];;
let value = 
  let lexeur = make_lexer ["+"; "-"; "*"; "/"; "("; ")"]
  and syntax_of_lexme = function
      | Kwd "+"  -> Infixe(  1,fun x y -> Bin("+", x,y))
      | Kwd "-"  -> InfPref((1,fun x y -> Bin("-", x,y)),(fun x -> Fct("opp",x)))
      | Kwd "*"  -> Infixe(  2,fun x y -> Bin("*", x,y))
      | Kwd "/"  -> Infixe(  2,fun x y -> Bin("/", x,y))
      | Int n    -> Valeur(Const(float_of_int n))
      | Float a  -> Valeur(Const a)
      | Ident i  -> if mem i fonctions   then Prfixe(fun x -> Fct(i,x))
                    else if i.[0] >= `A` then Valeur(Var i)
                    else raise Parse_error
      | Kwd "("  -> Lpar
      | Kwd ")"  -> Rpar
      | _        -> raise Parse_error
  in let eval = analyseur (fun () -> (2,fun x y -> Bin("*",x,y)))
                          syntax_of_lexme lexeur
  in function s -> try eval s
                   with Erreur(ex,i) -> print_erreur s i; raise ex
;;
#close "genlex";;
